/* jshint esversion : 6 */

(function() {

    const { areEquals } = $app;


    if (!Array.prototype.sortBy) {

        Object.defineProperty(Array.prototype, 'sortBy', {
            value: function(f) {
                for (let i = this.length; i;) {
                    const o = this[--i];
                    this[i] = [].concat(f.call(o, o, i), o);
                }
                this.sort(function(a, b) {
                    for (let i = 0, len = a.length; i < len; ++i) {
                        if (a[i] != b[i])
                            return a[i] < b[i] ? -1 : 1;
                    }
                    return 0;
                });
                for (let i = this.length; i;) {
                    this[--i] = this[i][this[i].length - 1];
                }
                return this;
            }
        });


    }


    if (!Array.prototype.unique) {
        Object.defineProperty(Array.prototype, 'unique', {
            value: function() {

                return this.reduce((curr, next) => {
                    const i = curr.find(e => areEquals(e, next));

                    if ("undefined" === typeof i || i == null)
                        curr.push(next);

                    return curr;
                }, []);

            }
        });
    }


    if (!Array.prototype.merge) {
        /**
         * @method {Array} merge
         */
        Object.defineProperty(Array.prototype, 'merge', {
            value: function merge(...a) {
                const x = Array.from(a).reduce((curr, next) => {
                    if (Array.isArray(next))
                        return curr.concat(next);
                    else {
                        curr.push(next);
                        return curr;
                    }
                }, this);

                for (let i = 0; i < x.length; i++) {
                    this.splice(i, 1, x[i]);
                }

                return this;
            }
        });

    }


    if (!Array.prototype.chunk) {

        Object.defineProperty(Array.prototype, 'chunk', {
            value: function(size, preserveKey) {

                const a = this;

                const l = a.length;
                const diff = l % size;
                const steps = diff ? ((l - diff) / size + 1) : (l / size);


                const _newArray = new Array(steps);

                for (let i = 0; i < steps; i++)
                    _newArray[i] = new Array(
                        (diff && i === (steps - 1)) ? diff : size
                    );


                let check = 0;
                let step = 0;
                for (let i = 0; i < l; i++) {
                    _newArray[step][preserveKey ? i : check] = a[i];
                    (check < size - 1) ? check++ : (check = 0, step++);
                }

                return _newArray;

            }
        });


    }




    if (!Array.prototype.groupBy) {


        Object.defineProperty(Array.prototype, 'groupBy', {
            value: function(field, returnArray) {

                if (!this.length)
                    return [];
                else {
                    const setGroupIdx = e => {
                        if ("string" === typeof field)
                            return e[field];
                        else if (Array.isArray(field)) {
                            const o = JSON.parse(JSON.stringify(e, field));
                            return Object.values(o).join();
                        }
                    };

                    const m = this.map(e => {
                        e.groupBy = setGroupIdx(e);
                        return e;
                    }).reduce((curr, next) => {
                        if (!curr[next.groupBy])
                            curr[next.groupBy] = [];

                        curr[next.groupBy].push(next);

                        curr[next.groupBy].map(e => {
                            delete e.groupBy;
                            return e;
                        });

                        return curr;

                    }, {});


                    if (!returnArray)
                        return m;
                    else
                        return Object.values(m);

                }
            }
        });




    }


}());